@@ -152,6 +152,20 @@ class System(pc.System):
 
         self.set_atoms(atoms)
 
+    def add_atoms(self, atoms):
+        """
+        Add a given list of atoms
+
+        Parameters
+        ----------
+        atoms : List of Atoms
+
+        Returns
+        -------
+        None
+        """
+        self.cadd_atoms(atoms)
+
     def read_inputfile(self, filename, format="lammps-dump", compressed = False, customkeys=None):
         """
 
@@ -399,9 +413,10 @@ class System(pc.System):
         threshold : float, optional
             only used if ``cutoff=adaptive``. A threshold which is used as safe limit for calculation of cutoff.
 
-        filter : {'None', 'type'}, optional
+        filter : {'None', 'type', 'type_r'}, optional
             apply a filter to nearest neighbor calculation. If the `filter` keyword is set to
-            `type`, only atoms of the same type would be included in the neighbor calculations. Default None.
+            `type`, only atoms of the same type would be included in the neighbor calculations. 
+            If `type_r`, only atoms of a different type will be included in the calculation. Default None.
 
         voroexp : int, optional
             only used if ``method=voronoi``. Power of the neighbor weight used to weight the contribution of each atom towards
@@ -483,6 +498,9 @@ class System(pc.System):
         if filter == 'type':
             # type corresponds to 1
             self.filter = 1
+        if filter == 'type_r':
+            # type corresponds to 1
+            self.filter = 2
 
         if method == 'cutoff':
             if cutoff=='sann':
@@ -1888,6 +1906,168 @@ class System(pc.System):
         self.box = newbox
         #self.atoms = atoms
 
+    def extract_cubic_box(self, repeat=(3,3,3)):
+        """
+        Extract a cubic representation of the box from triclinic cell
+
+        Parameters
+        ----------
+        repeat : list of ints
+            the number of times box should be repeat
+
+        Returns
+        -------
+        cubebox: list of list of floats
+            cubic box
+
+        atoms: list of Atom objects
+            atoms in the cubic box
+        """
+        ratoms = self.repeat(repeat)
+        anchor = ratoms[0]
+
+        nb = []
+
+        for count, ind in enumerate([[1, 2], [0, 2], [0, 1]]): 
+            mindist = 10000
+            minatom = None
+            for atom in ratoms[1:]:
+                if np.abs(atom.pos[ind[0]]-anchor.pos[ind[0]])< 1E-5:
+                    if np.abs(atom.pos[ind[1]]-anchor.pos[ind[1]])< 1E-5:
+                        if atom.type == anchor.type:
+                            dist = self.get_distance(anchor, atom)
+                            if dist < mindist:
+                                mindist = dist
+                                minatom = atom
+            if minatom is None:
+                raise ValueError("Cubic box cound not be found, try increasing repeat")
+            lo = min(anchor.pos[count], minatom.pos[count])
+            hi = max(anchor.pos[count], minatom.pos[count])
+            nb.append([lo, hi])
+
+        #collect atoms
+        collectedatoms = []
+        for atom in ratoms:
+            pos = atom.pos
+            if (nb[0][0] <= pos[0] < nb[0][1]):
+                if (nb[1][0] <= pos[1] < nb[1][1]):
+                    if (nb[2][0] <= pos[2] < nb[2][1]):
+                        collectedatoms.append(atom)
+
+        #remap atoms to box
+        for atom in collectedatoms:
+            pos = atom.pos
+            pos[0] = pos[0]-nb[0][0]
+            pos[1] = pos[1]-nb[1][0]
+            pos[2] = pos[2]-nb[2][0]
+            atom.pos = pos
+
+        #create a new box
+        cubebox = [[(nb[0][1]-nb[0][0]), 0, 0],
+                       [0, (nb[1][1]-nb[1][0]), 0],
+                       [0, 0, (nb[2][1]-nb[2][0])]]
+
+        return cubebox, collectedatoms
+
+
+    def get_concentration(self):
+        """
+        Return a dict containing the concentration of the system
+
+        Parameters
+        ----------
+        None
+
+        Returns
+        -------
+        condict : dict
+            dict of concentration values
+        """
+        typelist = [atom.type for atom in self.iter_atoms()]
+        types, typecounts = np.unique(typelist, return_counts=True)
+        concdict = {}
+        for c, t in enumerate(types):
+            concdict[str(t)] = typecounts[c]
+        return concdict
+
+    def remap_atoms(self, remove_images=True, assign=True, 
+        remove_atoms=False, dtol=0.1):
+        """
+        Remap atom back into simulation box
+
+        Parameters
+        ----------
+        pbc : bool, optional
+            If True, remove atoms on borders that are repeated
+
+        assign : bool, optional
+            If True, assign atoms to the system, otherwise return.
+
+        remove_atoms : bool, optional
+            If True, after the atoms, are remapped, remove those still 
+            outside the box. 
+
+        rtol : float, optional
+            Tolerance for removing atomic positions. Defaullt 0.1
+
+        """
+        box = self.box
+        atoms = self.atoms
+        self.box = box
+        newatoms = []
+
+        for atom in atoms:
+            pos = atom.pos
+            if pos[0] < 0:
+                pos[0] = pos[0]+box[0][0]
+            elif pos[0] >= box[0][0]:
+                pos[0] = pos[0]-box[0][0]
+            if pos[1] < 0:
+                pos[1] = pos[1]+box[1][1]
+            elif pos[1] >= box[1][1]:
+                pos[1] = pos[1]-box[1][1]
+            if pos[2] < 0:
+                pos[2] = pos[2]+box[2][2]
+            elif pos[2] >= box[2][2]:
+                pos[2] = pos[2]-box[2][2]
+            atom.pos = pos
+            
+        if remove_atoms:
+            temp = []
+            for atom in atoms:
+                pos = atom.pos
+                if (0 <= pos[0] < box[0][0]):
+                    if (0 <= pos[1] < box[1][1]):
+                        if (0 <= pos[2] < box[2][2]):
+                            temp.append(atom)
+            atoms = temp
+        
+        if remove_images:
+            pairs = []
+            for i in range(len(atoms)):
+                for j in range(i+1, len(atoms)):
+                    #we need explicit treatment of images
+                    dist = self.get_distance(atoms[i], atoms[j])
+                    if dist <= dtol:
+                        pairs.append(i) 
+            for count, atom in enumerate(atoms):
+                if count not in pairs:
+                    newatoms.append(atom)
+            if assign:
+                self.set_atoms(newatoms)
+                return None
+            else:
+                return newatoms
+
+        if assign:
+            self.set_atoms(atoms)
+            return None
+        else:
+            return atoms
+
+
+
+
     def show(self, colorby=None, filterby=None):
         """
         Plot the system
